<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

namespace Vvveb\Plugins\Seo\Controller;

use function Vvveb\__;
use Vvveb\Controller\Base;
use function Vvveb\getSetting;
use function Vvveb\setMultiSetting;
use Vvveb\System\CacheManager;
use Vvveb\System\Event;
use Vvveb\System\Sites;
use function Vvveb\url;

class Settings extends Base {
	protected $defaultTypes = ['post' => [
		'post' => [
			'name'    => 'Post',
			'source'  => 'default',
			'site_id' => '0',
			'type'    => 'post',
			'plural'  => 'posts',
			'icon'    => 'icon-document-text-outline',
			'comments'=> true,
		],
		'page' => [
			'name'    => 'Page',
			'source'  => 'default',
			'site_id' => '0',
			'type'    => 'page',
			'plural'  => 'pages',
			'icon'    => 'icon-document-outline',
			'comments'=> false,
		],
	],
		'product' => [
			'product' => [
				'name'    => 'Product',
				'type'    => 'product',
				'site_id' => '0',
				'source'  => 'default',
				'plural'  => 'products',
				'icon'    => 'icon-cube-outline',
			],
		],
	];

	function save() {
		//$validator = new Validator(['plugins.gravatar.settings']);
		$settings  = $this->request->post['settings'] ?? false;
		$errors    = [];

		if ($settings /*&&
			($errors = $validator->validate($settings)) === true*/) {
			//$settings              = $validator->filter($settings);
			$results               = setMultiSetting('seo', $settings);
			//CacheManager::delete('component');
			CacheManager::delete();
			$this->view->success[] = __('Settings saved!');
		} else {
			$this->view->validationErrors = $errors;
		}

		$this->index();
	}

	function feed($type = 'post') {
		$feed  = ['sitemap'=>[], 'feed' => []];
		$theme = Sites::getTheme() ?? 'default';

		foreach ([$theme, 'default'] as $t) {
			$dir   = DIR_THEMES . $t . DS . 'feed';

			if (is_dir($dir)) {
				$files = scandir($dir);

				foreach ($files as $file) {
					if ($file[0] == '.' || strpos($file, '.xml') == false) {
						continue;
					}
					$key  = "$t/$file";
					$slug = str_replace('.xml', '', basename($file));
					$url  = url('feed/index', ['rss' => $slug]);

					if (strpos($file, '-sitemap.xml') != false) {
						$ns = 'sitemap';
					} else {
						$ns = 'feed';
					}

					$feed[$ns][$key] = compact('slug', 'url', 'file');
				}
			}
		}

		return $feed;
	}

	function postTypes($type = 'post') {
		$typeName          = ucfirst($type);
		list($pluginTypes) = Event::trigger('Vvveb\Controller\Base', "custom$typeName", []);
		array_walk($pluginTypes, function (&$type,$key) {$type['source'] = 'plugin'; $type['name'] = ucfirst($key); });

		$userTypes = \Vvveb\getSetting($type, 'types', []);

		return $types = $this->defaultTypes[$type] + $pluginTypes + $userTypes;
	}

	function index() {
		$seo    = getSetting('seo') ?? [];
		$robots = DIR_PUBLIC . 'vrobots.txt';

		if (file_exists($robots)) {
			$seo['robots']['value'] = file_get_contents(DIR_PUBLIC . 'vrobots.txt') ?? '';
		}

		$seo['post-type']    = ($seo['post-type'] ?? []) + $this->postTypes('post');
		$seo['product-type'] = ($seo['product-type'] ?? []) + $this->postTypes('product');
		
		$this->view->schema = ['article' => 'Article', 'product' => 'Product', 'recipe' => 'Recipe'];

		//$feed = $this->feed();
		$seo += $this->feed();

		$this->view->seo = $seo;
	}
}
